@@ -15,6 +15,16 @@
 
 
 
+#if !defined(ARMA_WARN_LEVEL)
+  #define ARMA_WARN_LEVEL 2
+#endif
+//// The level of warning messages printed to ARMA_CERR_STREAM.
+//// Must be an integer >= 0. The default value is 2.
+//// 0 = no warnings
+//// 1 = only critical warnings about arguments and/or data which are likely to lead to incorrect results
+//// 2 = as per level 1, and warnings about poorly conditioned systems (low rcond) detected by solve(), spsolve(), etc
+//// 3 = as per level 2, and warnings about failed decompositions, failed saving/loading, etc
+
 #if !defined(ARMA_USE_LAPACK)
 #define ARMA_USE_LAPACK
 //// Comment out the above line if you don't have LAPACK or a high-speed replacement for LAPACK,
@@ -154,13 +164,13 @@
 //// change the number to the size of your vectors.
 
 #if !defined(ARMA_OPENMP_THRESHOLD)
-  #define ARMA_OPENMP_THRESHOLD 240
+  #define ARMA_OPENMP_THRESHOLD 320
 #endif
 //// The minimum number of elements in a matrix to allow OpenMP based parallelisation;
 //// it must be an integer that is at least 1.
 
 #if !defined(ARMA_OPENMP_THREADS)
-  #define ARMA_OPENMP_THREADS 10
+  #define ARMA_OPENMP_THREADS 8
 #endif
 //// The maximum number of threads to use for OpenMP based parallelisation;
 //// it must be an integer that is at least 1.
@@ -207,6 +217,11 @@
 //// Comment out the above line if you don't want errors and warnings printed (eg. failed decompositions)
 #endif
 
+#if !defined(ARMA_PRINT_EXCEPTIONS)
+// #define ARMA_PRINT_EXCEPTIONS
+//// see also compiler_setup.hpp
+#endif
+
 #if !defined(ARMA_PRINT_HDF5_ERRORS)
 // #define ARMA_PRINT_HDF5_ERRORS
 #endif
@@ -301,6 +316,15 @@
   #undef ARMA_PRINT_ERRORS
 #endif
 
+#if defined(ARMA_DONT_PRINT_EXCEPTIONS)
+  #undef ARMA_PRINT_EXCEPTIONS
+#endif
+
+#if !defined(ARMA_DONT_ZERO_INIT)
+  // #define ARMA_DONT_ZERO_INIT
+  //// Uncomment the above line to disable initialising elements to zero during construction of dense matrices and cubes
+#endif
+
 #if defined(ARMA_DONT_PRINT_HDF5_ERRORS)
   #undef ARMA_PRINT_HDF5_ERRORS
 #endif
