@@ -20,23 +20,23 @@
 
 
 //! conversion from Armadillo Base and BaseCube objects to scalars
-//! (kept only for compatibility with old code; use as_scalar() instead for Base objects like Mat)
+//! NOTE: use as_scalar() instead; this functionality is kept only for compatibility with old user code
 template<typename out_eT>
 class conv_to
   {
   public:
   
   template<typename in_eT, typename T1>
-  inline static out_eT from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static out_eT from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
 
   template<typename in_eT, typename T1>
-  inline static out_eT from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static out_eT from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static out_eT from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static out_eT from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static out_eT from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static out_eT from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -140,10 +140,10 @@ class conv_to< Mat<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static Mat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Mat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static Mat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Mat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   
   template<typename T1>
   inline static Mat<out_eT> from(const SpBase<out_eT, T1>& in);
@@ -151,10 +151,10 @@ class conv_to< Mat<out_eT> >
   
   
   template<typename in_eT>
-  inline static Mat<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Mat<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT>
-  inline static Mat<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Mat<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -172,7 +172,7 @@ conv_to< Mat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_
   const quasi_unwrap<T1> tmp(in.get_ref());
   const Mat<in_eT>& X  = tmp.M;
   
-  Mat<out_eT> out(X.n_rows, X.n_cols);
+  Mat<out_eT> out(X.n_rows, X.n_cols, arma_nozeros_indicator());
   
   arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
@@ -194,7 +194,7 @@ conv_to< Mat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_o
   const quasi_unwrap<T1> tmp(in.get_ref());
   const Mat<in_eT>& X  = tmp.M;
   
-  Mat<out_eT> out(X.n_rows, X.n_cols);
+  Mat<out_eT> out(X.n_rows, X.n_cols, arma_nozeros_indicator());
   
   arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
@@ -229,7 +229,7 @@ conv_to< Mat<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_n
   
   const uword N = uword( in.size() );
   
-  Mat<out_eT> out(N, 1);
+  Mat<out_eT> out(N, 1, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -253,7 +253,7 @@ conv_to< Mat<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
   
   const uword N = uword( in.size() );
   
-  Mat<out_eT> out(N, 1);
+  Mat<out_eT> out(N, 1, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -272,18 +272,18 @@ class conv_to< Row<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static Row<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Row<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static Row<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Row<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   
   
   
   template<typename in_eT>
-  inline static Row<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Row<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT>
-  inline static Row<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Row<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -303,7 +303,7 @@ conv_to< Row<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_
   
   arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
   
-  Row<out_eT> out(X.n_elem);
+  Row<out_eT> out(X.n_elem, arma_nozeros_indicator());
   
   arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
@@ -327,7 +327,7 @@ conv_to< Row<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_o
   
   arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
   
-  Row<out_eT> out(X.n_rows, X.n_cols);
+  Row<out_eT> out(X.n_rows, X.n_cols, arma_nozeros_indicator());
   
   arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
@@ -348,7 +348,7 @@ conv_to< Row<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_n
   
   const uword N = uword( in.size() );
   
-  Row<out_eT> out(N);
+  Row<out_eT> out(N, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -372,7 +372,7 @@ conv_to< Row<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
   
   const uword N = uword( in.size() );
   
-  Row<out_eT> out(N);
+  Row<out_eT> out(N, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -391,18 +391,18 @@ class conv_to< Col<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static Col<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Col<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static Col<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Col<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   
   
   
   template<typename in_eT>
-  inline static Col<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Col<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT>
-  inline static Col<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Col<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -422,7 +422,7 @@ conv_to< Col<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_
   
   arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
   
-  Col<out_eT> out(X.n_elem);
+  Col<out_eT> out(X.n_elem, arma_nozeros_indicator());
   
   arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
@@ -446,7 +446,7 @@ conv_to< Col<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_o
   
   arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
   
-  Col<out_eT> out(X.n_rows, X.n_cols);
+  Col<out_eT> out(X.n_rows, X.n_cols, arma_nozeros_indicator());
   
   arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
@@ -467,7 +467,7 @@ conv_to< Col<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_n
   
   const uword N = uword( in.size() );
   
-  Col<out_eT> out(N);
+  Col<out_eT> out(N, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -491,7 +491,7 @@ conv_to< Col<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
   
   const uword N = uword( in.size() );
   
-  Col<out_eT> out(N);
+  Col<out_eT> out(N, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -503,17 +503,72 @@ conv_to< Col<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
 
 
 
+//! convert between SpMat types
 template<typename out_eT>
 class conv_to< SpMat<out_eT> >
   {
   public:
   
+  template<typename in_eT, typename T1>
+  inline static SpMat<out_eT> from(const SpBase<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
+  
+  template<typename in_eT, typename T1>
+  inline static SpMat<out_eT> from(const SpBase<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
+  
   template<typename T1>
   inline static SpMat<out_eT> from(const Base<out_eT, T1>& in);
   };
 
 
 
+template<typename out_eT>
+template<typename in_eT, typename T1>
+arma_warn_unused
+inline
+SpMat<out_eT>
+conv_to< SpMat<out_eT> >::from(const SpBase<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk)
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  const unwrap_spmat<T1>  tmp(in.get_ref());
+  const SpMat<in_eT>& X = tmp.M;
+  
+  SpMat<out_eT> out(arma_layout_indicator(), X);
+  
+  arrayops::convert( access::rwp(out.values), X.values, X.n_nonzero );
+  
+  out.remove_zeros();
+  
+  return out;
+  }
+
+
+
+template<typename out_eT>
+template<typename in_eT, typename T1>
+arma_warn_unused
+inline
+SpMat<out_eT>
+conv_to< SpMat<out_eT> >::from(const SpBase<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk)
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  const unwrap_spmat<T1>  tmp(in.get_ref());
+  const SpMat<in_eT>& X = tmp.M;
+  
+  SpMat<out_eT> out(arma_layout_indicator(), X);
+  
+  arrayops::convert_cx( access::rwp(out.values), X.values, X.n_nonzero );
+  
+  out.remove_zeros();
+  
+  return out;
+  }
+
+
+
 template<typename out_eT>
 template<typename T1>
 arma_warn_unused
@@ -535,10 +590,10 @@ class conv_to< Cube<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static Cube<out_eT> from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Cube<out_eT> from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static Cube<out_eT> from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Cube<out_eT> from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -556,7 +611,7 @@ conv_to< Cube<out_eT> >::from(const BaseCube<in_eT, T1>& in, const typename arma
   const unwrap_cube<T1>  tmp( in.get_ref() );
   const Cube<in_eT>& X = tmp.M;
   
-  Cube<out_eT> out(X.n_rows, X.n_cols, X.n_slices);
+  Cube<out_eT> out(X.n_rows, X.n_cols, X.n_slices, arma_nozeros_indicator());
   
   arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
@@ -578,7 +633,7 @@ conv_to< Cube<out_eT> >::from(const BaseCube<in_eT, T1>& in, const typename arma
   const unwrap_cube<T1>  tmp( in.get_ref() );
   const Cube<in_eT>& X = tmp.M;
   
-  Cube<out_eT> out(X.n_rows, X.n_cols, X.n_slices);
+  Cube<out_eT> out(X.n_rows, X.n_cols, X.n_slices, arma_nozeros_indicator());
   
   arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
@@ -594,10 +649,10 @@ class conv_to< std::vector<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static std::vector<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static std::vector<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static std::vector<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static std::vector<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
